/* Software-based Trusted Platform Module (TPM) Emulator
 * Copyright (C) 2010 Stephen R. Tate <srtate@uncg.edu>
 *
 * This module is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This module is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#ifndef _TPM_TIMING_H
#define _TPM_TIMING_H

/*
 * The component times of the timing model:
 *   OVERC:    Constant overhead for a command execution
 *   OVERIN:   Overhead per byte of input
 *   OVEROUT:  Overhead per byte of output
 *   DECRYPT:  Time to do an RSA decryption
 *   ENCRYPT:  Time to do an RSA encryption
 *   SIGN:     Time to do an RSA signature
 *   HASHRATE: Hash rate (time per byte)
 */

enum TPMTimingOffsets { TPMTIMING_OVERC, TPMTIMING_OVERIN, TPMTIMING_OVEROUT, 
			TPMTIMING_DECRYPT, TPMTIMING_ENCRYPT, TPMTIMING_SIGN, 
			TPMTIMING_HASHRATE, TPMTIMING_FENCE };

/* Structure holding timing values */
struct TPMTimingConfig {
    const char *name;
    double value;
    int isSet;
};

extern int TPMTimingEnabled;

struct TPMTimingConfig TPMTimingOps[TPMTIMING_FENCE+1];

void TPMTiming_enable(void);
int TPMTiming_readConfig(const char *filename);

void TPMTiming_startDelay(void);
void TPMTiming_execDelay(void);

void TPMTiming_addDelay1(enum TPMTimingOffsets off);
void TPMTiming_addDelay2(enum TPMTimingOffsets off, int count);

/* 
 * To add an appropriate delay, the syntax is something like this:
 *    TPMTiming_addDelay2(TPMTIMING(TPMTIMING_HASHRATE), numBytes)
 */
#define TPMTIMING(off) (TPMTimingOps[off].value)

#endif
